// Copyright (c) 2006 David Drysdale.  See license.txt for license info
import java.applet.*;
import java.awt.*;

public class ablink extends Applet {
  // Widget vars
  LetterCanvas canvas;
  Button goButton;
  Button stopButton;
  Button clearButton;
  DisplayThread displayThread;
  boolean running = false;
  int[] totalCount;
  int[] missedCount;

  // Maths vars
  static final double xmax = 0.75;
  static final double ymax = 0.75;
  static final double timestep = 0.003;

  public String getAppletInfo() {
    return "ablink (c) David Drysdale 2006";
  }

  // Applet initialization
  public void init() {
    // Set up the applet layout
    Dimension d = new Dimension();
    d = size();
    Panel entryPanel = new Panel();
    goButton = new Button("Go");
    entryPanel.add(goButton);
    goButton.enable();
    stopButton = new Button("Stop");
    entryPanel.add(stopButton);
    stopButton.disable();
    clearButton = new Button("Clear Data");
    entryPanel.add(clearButton);
    clearButton.enable();
    this.add(entryPanel);
    canvas = new LetterCanvas();
    canvas.setSize(d.width, d.height - entryPanel.preferredSize().height - 10); 
    this.add(canvas, -1);
    // Store data
    totalCount = new int[20];
    missedCount = new int[20];
  }
  private void stopNow() {
    stopButton.disable();
    goButton.enable();
    running = false;
    displayThread.term();
    goButton.requestFocus();

    analyze(displayThread.data);
    canvas.displayData(missedCount, totalCount);
  }
  public boolean action(Event e, Object what) {
    boolean processed = false;
    if (e.target == goButton) {
      // Create the display thread
      canvas.clearData();
      displayThread = new DisplayThread(canvas);
      displayThread.setPriority(Thread.MIN_PRIORITY);
      canvas.paint(canvas.getGraphics());
      goButton.disable();
      stopButton.enable();
      canvas.requestFocus();
      running = true;
      displayThread.start();
    } else if (e.target == stopButton) {
      if (running) stopNow();
    } else if (e.target == clearButton) {
      for (int ii=0; ii<20; ii++) {
        totalCount[ii] = 0;
        missedCount[ii] = 0;
        synchronized (displayThread.data) {
          displayThread.data = "";
        }
      }
      canvas.clearData();
      canvas.requestFocus();
    }
    processed = true;
    return(processed);
  }
  
  public boolean keyDown(Event e, int key) {
    if (running) {
      if (key == Event.ESCAPE) {
        stopNow();
      } else {
        displayThread.mark();
      }
    }
    return true;
  }
  public void analyze(String dataString) {
    char[] c = dataString.toCharArray();
    System.out.println("Adding following raw data into statistics (uppercase indicates trigger, * a keypress)");
    System.out.println(dataString);
    int lastTrigger = -999;
    for (int ii=0; ii<(c.length-5); ii++) {
      if ((c[ii] == 'x') || (Character.isUpperCase(c[ii]))) {
        // This is a trigger char
        boolean spotted = false;
        int jj;
        for (jj = 1; jj <= 10; jj++) {
          if ((ii+jj) >= c.length) break;
          if (c[ii+jj] == '*') {
            spotted = true;
            c[ii+jj] = ' '; // Make sure it only counts once
            break;
          }
        }
         
        int distance = (ii - lastTrigger);
        if (distance < 20) {
          totalCount[distance]++;
          if (!spotted) missedCount[distance]++;
        }
        lastTrigger = ii;
      }
    }
    String results = "Delay between triggers, number of triggers, number of missed triggers\n";
    for (int dd=1; dd<20; dd++) {
      results += dd+", "+totalCount[dd]+", "+missedCount[dd]+"\n";
    }
    System.out.print("Statistics now (CSV format):");
    System.out.print(results);
  }
}
